@@ -18,23 +18,17 @@
 #' @name plm-deprecated
 #' @aliases detect_lin_dep
 #' @param formula a formula,
+#' @param x an object of class `"plm"`,
 #' @param lag.form a list containing the lag structure of each variable in the
 #' formula,
 #' @param diff.form a vector (or a list) of logical values indicating whether
 #' variables should be differenced,
 #' @param log.form a vector (or a list) of logical values indicating whether
 #' variables should be in logarithms.
-#' @param object,x an object of class `"plm"`,
 #' @param data a `data.frame`,
 #' @param \dots further arguments.
 #' @param indexes a vector (of length one or two) indicating the (individual
 #' and time) indexes (see Details);
-#' @param lhs see Formula
-#' @param rhs see Formula
-#' @param model see plm
-#' @param effect see plm
-#' @param theta the parameter of transformation for the random effect model
-#' @param cstcovar.rm remove the constant columns or not
 #' 
 NULL
 
@@ -47,14 +41,14 @@ pvcovHC <- function(x, ...){
 }
 
 
-# plm.data() is now deprecated (since February 2017). Need to keep it in package
+# plm.data() is deprecated since February 2017. Need to keep it in package
 # for backward compatibility of users' code out there and packages, especially 
 # for package 'systemfit' (systemfit supports pdata.frame since 2017-03-09 but 
-# plm.data can be used there as well)..
+# plm.data can be used there as well).
 #
-# While plm.data() was a 'full function' once, it now is now using
-# pdata.frame() and re-works the properties of the "plm.dim" objects
-# original created by the 'full' plm.data() function.  The 'full'
+# While plm.data() was a 'full function' once, it now uses
+# pdata.frame() and re-works the properties of the "plm.dim" object
+# original created by the 'full' plm.data() function. The 'full'
 # plm.data() function is kept non-exported as plm.data_depr_orig due
 # to reference and testing (see tests/test_plm.data.R)
 
@@ -272,13 +266,13 @@ pht <- function(formula, data, subset, na.action, model = c("ht", "am", "bms"),
   }
   
   X <- model.matrix(data, model = "pooling", rhs = 1, lhs = 1)
-  if (length(exo.var) > 0L) XV <- X[ , exo.var, drop = FALSE] else XV <- NULL
-  if (length(edo.var) > 0L) NV <- X[ , edo.var, drop = FALSE] else NV <- NULL
-  if (length(exo.cst) > 0L) XC <- X[ , exo.cst, drop = FALSE] else XC <- NULL
-  if (length(edo.cst) > 0L) NC <- X[ , edo.cst, drop = FALSE] else NC <- NULL
-  if (length(all.cst) != 0L)
-    zo <- twosls(fixef[as.character(id)], cbind(XC, NC), cbind(XC, XV), TRUE)
-  else zo <- lm(fixef ~ 1)
+  XV <- if(length(exo.var) > 0L) X[ , exo.var, drop = FALSE] else NULL
+  NV <- if(length(edo.var) > 0L) X[ , edo.var, drop = FALSE] else NULL
+  XC <- if(length(exo.cst) > 0L) X[ , exo.cst, drop = FALSE] else NULL
+  NC <- if(length(edo.cst) > 0L) X[ , edo.cst, drop = FALSE] else NULL
+  zo <- if(length(all.cst) != 0L) {
+    twosls.pht(fixef[as.character(id)], cbind(XC, NC), cbind(XC, XV), intercept = TRUE)
+  } else lm(fixef ~ 1)
   
   sigma2 <- list()
   sigma2$one <- 0
@@ -349,8 +343,8 @@ pht <- function(formula, data, subset, na.action, model = c("ht", "am", "bms"),
     }
     W <- cbind(within.inst, XC, between.inst, Vxstar)
   }
-  
-  result <- twosls(y, X, W)
+
+  result <- twosls.pht(y, X, W)
   K <- length(data)
   ve <- lev2var(data)
   varlist <- list(xv = unique(ve[exo.var]),
@@ -375,6 +369,49 @@ pht <- function(formula, data, subset, na.action, model = c("ht", "am", "bms"),
     result
 }
 
+twosls.pht <- function(y, X, W, intercept = FALSE, lm.type = "lm"){
+## this is a dedicated version of twosls() only for use in pht() as the
+## pht() code requires the instrument matrix W always be amended with an 
+## intercept prior to regression, see marked line below. It was easier to
+## have this separate dedicated twosls.pht() function than to adjust the 
+## rest of the pht() code.
+  
+  ## non-exported
+  # Return value can be controlled by argument lm.type. Often, a full lm model
+  # is needed for further processing but can select one of the fast but less
+  # rich objects produced by lm.fit or .lm.fit (the latter does not contain, e.g.,
+  # fitted.values and is to be used very carefully (e.g., coefs not in input order)).
+  
+  # As NA/NaN/(+/-)Inf-freeness needs to be guaranteed when functions call
+  # twosls(), so can use lm.fit to calc. Xhat.
+  
+  Xhat <- lm.fit(cbind(1, W), X)$fitted.values ##### this line is different relative to twosls()
+  
+  if(!is.matrix(Xhat)) {
+    # ensure Xhat is a matrix
+    Xhat <- matrix(Xhat, ncol = 1L)
+    colnames(Xhat) <- colnames(X)
+  }
+  
+  if(intercept) {
+    model <- switch(lm.type,
+                    "lm"      =  lm(y ~ Xhat),
+                    "lm.fit"  =  lm.fit(cbind(1, Xhat), y),
+                    ".lm.fit" = .lm.fit(cbind(1, Xhat), y))
+    yhat <- as.vector(crossprod(t(cbind(1, X)), coef(model)))
+  }
+  else{
+    model <- switch(lm.type,
+                    "lm"      =  lm(y ~ Xhat - 1),
+                    "lm.fit"  =  lm.fit(Xhat, y),
+                    ".lm.fit" = .lm.fit(Xhat, y))
+    yhat <- as.vector(crossprod(t(X), coef(model)))
+  }
+  model$residuals <- y - yhat
+  model
+}
+
+
 #' @rdname pht
 #' @export
 summary.pht <- function(object, ...){
@@ -452,16 +489,6 @@ print.summary.pht <- function(x, digits = max(3, getOption("digits") - 2),
 
 ## dynformula
 
-sumres <- function(x){
-    sr <- summary(unclass(resid(x)))
-    srm <- sr["Mean"]
-    if (abs(srm) < 1e-10){
-        sr <- sr[c(1:3, 5:6)]
-    }
-    sr
-}
-
-
 create.list <- function(alist, K, has.int, has.resp, endog, exo, default){
     # if alist is NULL, create a list of 0
     if (is.null(alist)) alist <- rep(list(default), K+has.resp)
@@ -512,7 +539,6 @@ write.lags <- function(name, lags, diff){
                 if (diff) chlag <- c(chlag, paste("diff(",name,")")) else chlag <- c(chlag,name)
             }
             else{
-                ichar <- paste(i)
                 chlag <- c(chlag, paste(lag.string,"(",name,",",i,")",sep=""))
             }
         }
@@ -603,127 +629,3 @@ formula.dynformula <- function(x, ...){
 print.dynformula <- function(x, ...){
     print(formula(x), ...)
 }
-
-#' @rdname plm-deprecated
-#' @export
-pFormula <- function(object) {
-  .Deprecated(msg = paste0("class 'pFormula' is deprecated, simply use class",
-                          "'Formula'. 'pFormula' will be removed very soon!"),
-              old = "pFormula", new = "Formula")
-  stopifnot(inherits(object, "formula"))
-  if (!inherits(object, "Formula")){
-    object <- Formula(object)
-  }
-  class(object) <- unique(c("pFormula", class(object)))
-  object
-}
-
-#' @rdname plm-deprecated
-#' @export
-as.Formula.pFormula <- function(x, ...){
-  class(x) <- setdiff(class(x), "pFormula")
-  x
-}
-
-
-## pFormula stuff, usefull for cquad
-
-#' @rdname plm-deprecated
-#' @export
-as.Formula.pFormula <- function(x, ...){
-    class(x) <- setdiff(class(x), "pFormula")
-    x
-}
-
-#' @rdname plm-deprecated
-#' @export
-model.frame.pFormula <- function(formula, data, ..., lhs = NULL, rhs = NULL){
-    if (is.null(rhs)) rhs <- 1:(length(formula)[2L])
-    if (is.null(lhs)) lhs <- if(length(formula)[1L] > 0L) 1 else 0
-    index <- attr(data, "index")
-    mf <- model.frame(as.Formula(formula), as.data.frame(data), ..., rhs = rhs)
-    index <- index[as.numeric(rownames(mf)), ]
-    index <- droplevels(index)
-    class(index) <- c("pindex", "data.frame")
-    structure(mf,
-              index = index,
-              class = c("pdata.frame", class(mf)))
-}
-
-
-#' @rdname plm-deprecated
-#' @export
-model.matrix.pFormula <- function(object, data,
-                                  model = c("pooling", "within", "Between", "Sum",
-                                            "between", "mean", "random", "fd"),
-                                  effect = c("individual", "time", "twoways", "nested"),
-                                  rhs = 1,
-                                  theta = NULL,
-                                  cstcovar.rm = NULL,
-                                  ...){
-    model <- match.arg(model)
-    effect <- match.arg(effect)
-    formula <- object  
-    has.intercept <- has.intercept(formula, rhs = rhs)
-    # relevant defaults for cstcovar.rm
-    if (is.null(cstcovar.rm)) cstcovar.rm <- ifelse(model == "within", "intercept", "none")
-    balanced <- is.pbalanced(data)
-    # check if inputted data is a model.frame, if not convert it to
-    # model.frame (important for NA handling of the original data when
-    # model.matrix.pFormula is called directly) As there is no own
-    # class for a model.frame, check if the 'terms' attribute is
-    # present (this mimics what lm does to detect a model.frame)    
-    if (is.null(attr(data, "terms")))
-        data <- model.frame.pFormula(pFormula(formula), data)  
-    # this goes to Formula::model.matrix.Formula:
-    X <- model.matrix(as.Formula(formula), rhs = rhs, data = data, ...)
-    # check for infinite or NA values and exit if there are some
-    if(any(! is.finite(X))) stop(paste("model matrix or response contains non-finite",
-                                       "values (NA/NaN/Inf/-Inf)"))
-    X.assi <- attr(X, "assign")
-    X.contr <- attr(X, "contrasts")
-    X.contr <- X.contr[ ! sapply(X.contr, is.null) ]
-    index <- index(data)
-    checkNA.index(index) # check for NAs in model.frame's index and error if any
-    attr(X, "index") <- index
-    if (effect == "twoways" && model %in% c("between", "fd"))
-        stop("twoways effect only relevant for within, random and pooling models")
-    if (model == "within")  X <- Within(X, effect)
-    if (model == "Sum")     X <- Sum(X, effect)
-    if (model == "Between") X <- Between(X, effect)
-    if (model == "between") X <- between(X, effect)
-    if (model == "mean")    X <- Mean(X)
-    if (model == "fd")      X <- pdiff(X, effect = "individual",
-                                       has.intercept = has.intercept)
-    if (model == "random"){
-        if (is.null(theta)) stop("a theta argument should be provided")
-        if (effect %in% c("time", "individual")) X <- X - theta * Between(X, effect)
-        if (effect == "nested") X <- X - theta$id * Between(X, "individual") -
-                                    theta$gp * Between(X, "group")
-        if (effect == "twoways" && balanced)
-            X <- X - theta$id * Between(X, "individual") -
-                theta$time * Between(X, "time") + theta$total * Mean(X)
-    }
-
-    if (cstcovar.rm == "intercept"){
-        posintercept <- match("(Intercept)", colnames(X))
-        if (! is.na(posintercept)) X <- X[ , - posintercept, drop = FALSE]
-    }
-    if (cstcovar.rm %in% c("covariates", "all")){
-        cols <- apply(X, 2, is.constant)
-        cstcol <- names(cols)[cols]
-        posintercept <- match("(Intercept)", cstcol)
-        cstintercept <- if(is.na(posintercept)) FALSE else TRUE
-        zeroint <- if(cstintercept &&
-                          max(X[, posintercept]) < sqrt(.Machine$double.eps))
-                          TRUE else FALSE
-        if (length(cstcol) > 0L){
-            if ((cstcovar.rm == "covariates" || !zeroint) && cstintercept) cstcol <- cstcol[- posintercept]
-            if (length(cstcol) > 0L){
-                X <- X[, - match(cstcol, colnames(X)), drop = FALSE]
-                attr(X, "constant") <- cstcol
-            }
-        }
-    }
-    structure(X, assign = X.assi, contrasts = X.contr, index = index)
-}
