@@ -1,7 +1,9 @@
   ## Mean Group estimator
   ## ref. Coakley, Fuertes and Smith 2004
   ##
-  ## This version 10:
+  ## This version 11:
+  ##   uses optimised data split approach (via split())
+  ## version 10:
   ##   added R2 = 1-var(resid)/var(y) as a measure of fit
   ## from version 9:
   ##   fixed residuals
@@ -41,14 +43,14 @@
 #' common factors (CCEMG)
 #' 
 #' `pmg` is a function for the estimation of linear panel models with
-#' heterogeneous coefficients by the Mean Groups estimator. `model =
-#' "mg"` specifies the standard Mean Groups estimator, based on the
+#' heterogeneous coefficients by various Mean Groups estimators. Setting
+#' argument `model = "mg"` specifies the standard Mean Groups estimator, based on the
 #' average of individual time series regressions. If `model = "dmg"`
 #' the data are demeaned cross-sectionally, which is believed to
 #' reduce the influence of common factors (and is akin to what is done
-#' in homogeneous panels when `model = "within"` and `effect =
-#' "time"`). Lastly, if `model = "cmg"` the CCEMG estimator is
-#' employed: this latter is consistent under the hypothesis of
+#' in homogeneous panels when `model = "within"` and `effect = "time"`).
+#' Lastly, if `model = "cmg"` the CCEMG estimator is
+#' employed which is consistent under the hypothesis of
 #' unobserved common factors and idiosyncratic factor loadings; it
 #' works by augmenting the model by cross-sectional averages of the
 #' dependent variable and regressors in order to account for the
@@ -61,13 +63,14 @@
 #' @param data a `data.frame`,
 #' @param subset see [lm()],
 #' @param na.action see [lm()],
-#' @param model one of `c("mg", "cmg", "dmg")`,
+#' @param model one of `"mg"`, `"cmg"`, or `"dmg"`,
 #' @param index the indexes, see [pdata.frame()],
 #' @param trend logical specifying whether an individual-specific
 #'     trend has to be included,
 #' @param digits digits,
 #' @param width the maximum length of the lines in the print output,
 #' @param \dots further arguments.
+#' 
 #' @return An object of class `c("pmg", "panelmodel")` containing:
 #'     \item{coefficients}{the vector of coefficients,}
 #'     \item{residuals}{the vector of residuals,}
@@ -75,11 +78,11 @@
 #'     \item{vcov}{the covariance matrix of the coefficients,}
 #'     \item{df.residual}{degrees of freedom of the residuals,}
 #'     \item{model}{a data.frame containing the variables used for the
-#'     estimation,} \item{call}{the call,} \item{sigma}{always `NULL`,
-#'     `sigma` is here only for compatibility reasons (to allow using
-#'     the same `summary` and `print` methods as `pggls`),}
+#'                  estimation,}
+#'     \item{r.squared}{numeric, the R squared,}
+#'     \item{call}{the call,}
 #'     \item{indcoef}{the matrix of individual coefficients from
-#'     separate time series regressions.}
+#'                    separate time series regressions.}
 #' @export
 #' @author Giovanni Millo
 #' @references
@@ -101,11 +104,10 @@
 #' ## Common Correlated Effects Mean Groups
 #' ccemgmod <- pmg(log(gsp) ~ log(pcap) + log(pc) + log(emp) + unemp, 
 #'                 data = Produc, model = "cmg")
-#' summary(ccemgmod) 
+#' summary(ccemgmod)
 pmg <- function(formula, data, subset, na.action,
                 model = c("mg", "cmg", "dmg"), index = NULL,
-                trend = FALSE, ...)
-{
+                trend = FALSE, ...) {
 
     ## same as pggls but for effect, fixed at "individual" for compatibility
     ## ind for id, tind for time, k for K, coefnam for coef.names
@@ -126,31 +128,30 @@ pmg <- function(formula, data, subset, na.action,
     ## evaluates the call, modified with model = "pooling", inside the
     ## parent frame resulting in the pooling model on formula, data
     plm.model <- eval(plm.model, parent.frame())
-    index <- attr(model.frame(plm.model), "index")
+    mf <- model.frame(plm.model)
+    index <- unclass(attr(mf, "index")) # unclass for speed
     ind  <- index[[1L]] ## individual index
+    ind.GRP <- collapse::GRP(ind) ## individual index as collapse-optimised GRP
     tind <- index[[2L]] ## time index
     ## set dimension variables
     pdim <- pdim(plm.model)
-    balanced <- pdim$balanced
-    nt <- pdim$Tint$nt
     Ti <- pdim$Tint$Ti
     T. <- pdim$nT$T
     n <- pdim$nT$n
-    N <- pdim$nT$N
     ## set index names
     time.names <- pdim$panel.names$time.names
-    id.names <- pdim$panel.names$id.names
+    id.names   <- pdim$panel.names$id.names
     coef.names <- names(coef(plm.model))
     ## number of coefficients
     k <- length(coef.names)
 
     ## model data
     X <- model.matrix(plm.model)
-    y <- model.response(model.frame(plm.model))
+    y <- model.response(mf)
 
 
   ## det. *minimum* group numerosity
-  t <- min(Ti) # == min(tapply(X[ , 1], ind, length))
+  t <- min(Ti)
 
   ## check min. t numerosity
   ## NB it is also possible to allow estimation if there *is* one group
@@ -164,19 +165,25 @@ pmg <- function(formula, data, subset, na.action,
   ## (might be unbalanced => t1!=t2 but we don't care as long
   ## as min(t)>k+1)
 
-  ## "pre-allocate" coefficients matrix for the n models
-  kt <- if (trend) 1L else 0L
-  tcoef <- matrix(data = NA_real_, nrow = k+kt, ncol = n)
+  ## "pre-allocate" coefficients matrix for the n models and list for residuals
+  kt <- if(trend) 1L else 0L
+  tcoef <- if(model == "cmg") {
+    matrix(data = NA_real_, nrow = 2*k+kt, ncol = n) ## allow for extended coef vector
+  } else matrix(data = NA_real_, nrow = k+kt, ncol = n)
   tres <- vector("list", n)
 
   switch(model,
     "mg" = {
+      # split X, y by individual and store in lists
+      tX.list <- collapse::rsplit(X, ind.GRP, use.names = FALSE)
+      
+      ty.list <- collapse::gsplit(y, ind.GRP)
+      
       ## for each x-sect. i = 1..n
-      unind <- unique(ind)
-      for(i in 1:n) {
-        tX <- X[ind == unind[i], ]
-        ty <- y[ind == unind[i]]
-        if(trend) tX <- cbind(tX, 1:(dim(tX)[[1L]]))
+      for(i in seq_len(n)) {
+        tX <- tX.list[[i]]
+        ty <- ty.list[[i]]
+        if(trend) tX <- cbind(tX, seq_len(dim(tX)[[1L]]))
         tfit <- lm.fit(tX, ty)
         tcoef[ , i] <- tfit$coefficients
         tres[[i]]   <- tfit$residuals
@@ -189,41 +196,31 @@ pmg <- function(formula, data, subset, na.action,
     
     "cmg" = {
       ## between-periods transformation (take means over groups for each t)
-         #  be <- function(x, index, na.rm = TRUE) tapply(x, index, mean, na.rm = na.rm)
-         #  Xm <- apply(X, 2, FUN = be, index = tind)[tind, , drop = FALSE]
-         #  ym <- apply(as.matrix(as.numeric(y)), 2, FUN = be, index = tind)[tind]
       Xm <- Between(X, effect = "time", na.rm = TRUE)
       ym <- as.numeric(Between(y, effect = "time", na.rm = TRUE))
       
-      augX <- cbind(X, ym, Xm[ , -1, drop = FALSE])
-
-      ## allow for extended coef vector
-      tcoef0 <- matrix(data = NA_real_, nrow = 2*k+kt, ncol = n)
+      augX <- cbind(X, ym, Xm[ , -1L, drop = FALSE])
 
+      taugX.list <- collapse::rsplit(augX, ind.GRP, use.names = FALSE)
+      
+      ty.list <- collapse::gsplit(y, ind.GRP)
+      
       ## for each x-sect. i = 1..n estimate (over t) an augmented model
       ## y_it = alpha_i + beta_i*X_it + c1_i*my_t + c2_i*mX_t + err_it
-      unind <- unique(ind)
-      for(i in 1:n) {
-        taugX <- augX[ind == unind[i], ]
-        ty <- y[ind == unind[i]]
-
-        if(trend) taugX <- cbind(taugX, 1:(dim(taugX)[[1L]]))
-
+      for(i in seq_len(n)) {
+        taugX <- taugX.list[[i]]
+        ty    <-    ty.list[[i]]
+        if(trend) taugX <- cbind(taugX, seq_len(dim(taugX)[[1L]]))
         tfit <- lm.fit(taugX, ty)
-        tcoef0[ , i] <- tfit$coefficients
-        tres[[i]]    <- tfit$residuals
+        tcoef[ , i] <- tfit$coefficients
+        tres[[i]]   <- tfit$residuals
       }
-      tcoef <- tcoef0[1:k, ]
-      tcoef.bar <- tcoef0[-(1:k), ]
 
+      ## add names of coefs for augmented x-sectional averages
       coef.names.bar <- c("y.bar", paste(coef.names[-1L], ".bar", sep=""))
-
-      ## 'trend' always comes last
-      if(trend) coef.names.bar <- c(coef.names.bar, "trend")
-
-      ## output complete coefs
-      tcoef <- tcoef0
       coef.names <- c(coef.names, coef.names.bar)
+      ## 'trend' always comes last
+      if(trend) coef.names <- c(coef.names, "trend")
       ## adjust k
       k <- length(coef.names)
 
@@ -231,28 +228,21 @@ pmg <- function(formula, data, subset, na.action,
       },
     
     "dmg" = {
-      ## old: between-periods transformation (take means over group for each t)
-         ##  be <- function(x, index, na.rm = TRUE) tapply(x, index, mean, na.rm = na.rm)
-         ##  Xm <- apply(X, 2, FUN = be, index = tind)[tind, , drop = FALSE]
-         ##  ym <- apply(as.matrix(as.numeric(y)), 2, FUN = be, index = tind)[tind]
-          # Xm <- Between(X, effect = "time", na.rm = TRUE)
-          # ym <- as.numeric(Between(y, effect = "time", na.rm = TRUE))
-          ## ...but of course we do not demean the intercept!
-          # Xm[ , 1] <- 0
-          # demX <- X - Xm
-          # demy <- y - ym
-      
+      ##  time-demean
       demX <- Within(X, effect = "time", na.rm = TRUE)
-      demX[ , 1] <- 1 # put back intercept lost by within transformation
+      demX[ , 1L] <- 1 # put back intercept lost by within transformation
       demy <- as.numeric(Within(y, effect = "time", na.rm = TRUE))
+
+      tdemX.list <- collapse::rsplit(demX, ind.GRP, use.name = FALSE)
       
+      tdemy.list <- collapse::gsplit(demy, ind.GRP)
+            
       ## for each x-sect. i=1..n estimate (over t) a demeaned model
       ## (y_it-my_t) = alpha_i + beta_i*(X_it-mX_t) + err_it
-      unind <- unique(ind)
-      for (i in 1:n) {
-        tdemX <- demX[ind == unind[i], ]
-        tdemy <- demy[ind == unind[i]]
-        if(trend) tdemX <- cbind(tdemX, 1:(dim(tdemX)[[1L]]))
+      for (i in seq_len(n)) {
+        tdemX <- tdemX.list[[i]]
+        tdemy <- tdemy.list[[i]]
+        if(trend) tdemX <- cbind(tdemX, seq_len(dim(tdemX)[[1L]]))
         tfit <- lm.fit(tdemX, tdemy)
         tcoef[ , i] <- tfit$coefficients
         tres[[i]]   <- tfit$residuals
@@ -270,10 +260,10 @@ pmg <- function(formula, data, subset, na.action,
     coefmat <- array(data = NA_real_, dim = c(k, k, n))
     demcoef <- tcoef - coef # gets recycled n times by column
 
-    for (i in 1:n) coefmat[ , , i] <- outer(demcoef[ , i], demcoef[ , i])
+    for (i in seq_len(n)) coefmat[ , , i] <- outer(demcoef[ , i], demcoef[ , i])
     ## summing over the n-dimension of the array we get the
     ## covariance matrix of coefs
-    vcov <- apply(coefmat, 1:2, sum)/(n*(n-1))
+    vcov <- rowSums(coefmat, dims = 2L) / (n*(n-1)) # == apply(coefmat, 1:2, sum) / (n*(n-1)) but rowSums(., dims = 2L)-construct is way faster
 
     ######### na.omit = T in apply was the big problem!!
 
@@ -290,14 +280,17 @@ pmg <- function(formula, data, subset, na.action,
 
     names(coef) <- rownames(vcov) <- colnames(vcov) <- coef.names
     dimnames(tcoef) <- list(coef.names, id.names)
-    pmodel <- attr(plm.model, "pmodel")
-    pmodel$model.name <- model.name
-    mgmod <- list(coefficients = coef, residuals = residuals,
-                  fitted.values = fitted.values, vcov = vcov,
-                  df.residual = df.residual, r.squared = r2,
-                  model = model.frame(plm.model), sigma = NULL,
-                  indcoef = tcoef, formula = formula,
-                  call = cl)
+    pmodel <- list(model.name = model.name)
+    mgmod <- list(coefficients  = coef,
+                  residuals     = residuals,
+                  fitted.values = fitted.values,
+                  vcov          = vcov,
+                  df.residual   = df.residual,
+                  r.squared     = r2,
+                  model         = mf,
+                  indcoef       = tcoef,
+                  formula       = formula,
+                  call          = cl)
     mgmod <- structure(mgmod, pdim = pdim, pmodel = pmodel)
     class(mgmod) <- c("pmg", "panelmodel")
     mgmod
@@ -306,7 +299,6 @@ pmg <- function(formula, data, subset, na.action,
 #' @rdname pmg
 #' @export
 summary.pmg <- function(object, ...){
-  pmodel <- attr(object, "pmodel")
   std.err <- sqrt(diag(object$vcov))
   b <- object$coefficients
   z <- b/std.err
@@ -316,7 +308,7 @@ summary.pmg <- function(object, ...){
   object$CoefTable <- CoefTable
   y <- object$model[[1L]]
   object$tss <- tss(y)
-  object$ssr <- sum(residuals(object)^2)
+  object$ssr <- as.numeric(crossprod(residuals(object)))
   object$rsqr <- 1-object$ssr/object$tss
   class(object) <- c("summary.pmg")
   return(object)
@@ -324,28 +316,27 @@ summary.pmg <- function(object, ...){
 
 #' @rdname pmg
 #' @export
-print.summary.pmg <- function(x, digits = max(3, getOption("digits") - 2), width = getOption("width"), ...){
+print.summary.pmg <- function(x, digits = max(3, getOption("digits") - 2),
+                              width = getOption("width"), ...){
   pmodel <- attr(x, "pmodel")
-  pdim <- attr(x, "pdim")
-#  formula <- pmodel$formula
-  model.name <- pmodel$model.name
-  cat(paste(model.pmg.list[model.name], "\n", sep=""))
+  pdim   <- attr(x, "pdim")
+  cat(paste(model.pmg.list[pmodel$model.name], "\n", sep=""))
   cat("\nCall:\n")
   print(x$call)
   cat("\n")
   print(pdim)
   cat("\nResiduals:\n")
-  print(summary(unlist(residuals(x))))
+  print(sumres(x))
   cat("\nCoefficients:\n")
   printCoefmat(x$CoefTable, digits = digits)
-  cat(paste("Total Sum of Squares: ",    signif(x$tss, digits), "\n", sep=""))
-  cat(paste("Residual Sum of Squares: ", signif(x$ssr, digits), "\n", sep=""))
-  cat(paste("Multiple R-squared: ",      signif(x$rsqr, digits),"\n", sep=""))
+  cat(paste("Total Sum of Squares: ",    signif(x$tss,  digits),  "\n", sep=""))
+  cat(paste("Residual Sum of Squares: ", signif(x$ssr,  digits),  "\n", sep=""))
+  cat(paste("Multiple R-squared: ",      signif(x$rsqr, digits), "\n", sep=""))
   invisible(x)
 }
 
 #' @rdname pmg
 #' @export
 residuals.pmg <- function(object, ...) {
-  return(pres(object))
+  pres(object)
 }
