@@ -63,7 +63,8 @@ aneweytest <- function(formula, data, subset, na.action, index = NULL,  ...){
     ht$effect <- "individual"
     ht <- eval(ht, parent.frame())
     
-    .resid <- split(resid(ht), time)
+    time.GRP <- collapse::GRP(time)
+    .resid <- collapse::gsplit(resid(ht), time.GRP)
 
     # extract the covariates (no intercept), and isolate time-invariant covariates
     X <- model.matrix(data, model = "pooling", rhs = 1, lhs = 1)[ , -1, drop = FALSE]
@@ -88,10 +89,13 @@ aneweytest <- function(formula, data, subset, na.action, index = NULL,  ...){
     # time-demean and split by period:
     attr(X, "index") <- index
     X <- Within(X, effect ="time")
-    X <- lapply(as.list(periods), function(x) X[time == x, , drop = FALSE])
+    namesX <- colnames(X)
+    
+    X <- collapse::rsplit(X, time.GRP, use.names = FALSE)
+    
     # put column names for split matrices in X:
-    for (i in 1:(length(periods))){
-      colnames(X[[i]]) <- paste(colnames(X[[i]]), periods[i], sep = ".")
+    for (i in seq_along(periods)){
+      colnames(X[[i]]) <- paste(namesX, periods[i], sep = ".")
     }
     
     if (!is.null(Z)){
@@ -186,7 +190,8 @@ piest <- function(formula, data, subset, na.action, index = NULL, robust = TRUE,
     # extract the response, time-demean and split by period
     y <- pmodel.response(data, model = "pooling", effect = "individual")
     Y <- Within(y, "time")
-    Y <- split(Y, time)
+    time.GRP <- collapse::GRP(time)
+    Y <- collapse::gsplit(Y, time.GRP)
     
     # extract the covariates, and isolate time-invariant covariates
     X <- model.matrix(data, model = "pooling", rhs = 1, lhs = 1)[ , -1, drop = FALSE]
@@ -213,10 +218,12 @@ piest <- function(formula, data, subset, na.action, index = NULL, robust = TRUE,
     attr(X, "index") <- index
     X <- Within(X, effect ="time")
     periods <- unique(time)
-    X <- lapply(as.list(periods), function(x) X[time == x, , drop = FALSE])
-    # put columnnames for split matrices in X:
-    for (i in 1:(length(periods))){
-      colnames(X[[i]]) <- paste(colnames(X[[i]]), periods[i], sep = ".")
+    
+    X <- collapse::rsplit(X, time.GRP, use.names = FALSE)
+
+    # put column names for split matrices in X:
+    for (i in seq_along(periods)){
+      colnames(X[[i]]) <- paste(namesX, periods[i], sep = ".")
     }
     
     if (!is.null(Z)){
@@ -228,7 +235,7 @@ piest <- function(formula, data, subset, na.action, index = NULL, robust = TRUE,
     
     # compute the unconstrained estimates
       # NA-freeness guaranteed by model frame construction, so can use lm.fit
-      # (non-collinearity is not cared for but code error if collinearity is 
+      # (non-collinearity is not cared for but code errors if collinearity is 
       # present anyway a bit later)
       #   was:   LMS <- lapply(Y, function(x) lm(x ~ XX - 1))
     LMS <- lapply(Y, function(x) lm.fit(XX, x))
@@ -252,15 +259,15 @@ piest <- function(formula, data, subset, na.action, index = NULL, robust = TRUE,
     
     # construct the matrix of linear restrictions R | R x theta = pi
     R <- matrix(0, T * (T * Kx + Kz), (T + 1) * Kx + Kz)
-    for (i in 1:Kx){
+    for (i in seq_len(Kx)){
         R[ ((1:T) - 1) * (Kx * T + Kz) + (Kx * (1:T - 1)) + i , i] <- 1
     }
     if (Kz > 0){
-        for (i in 1:Kz){
+        for (i in seq_len(Kz)){
             R[ (Kx * T) + (1:T - 1) * (Kx * T + Kz) + i, Kx + i] <- 1
         }
     }
-    for (i in 1:(Kx * T)){
+    for (i in seq_len(Kx * T)){
         R[((1:T) - 1) * (Kx * T + Kz) + i , Kx + Kz + i] <- 1
     }
     
